/*******************************************************
**
** Definition of QyurSqlTreeView
**
** Copyright (C) 2009-2013 Yuri Yurachkovsky.
**
** QyurSqlTreeView is free software; you can redistribute
** it and/or modify it under the terms of the GNU
** Library General Public License as published by the
** Free Software Foundation; either version 2, or (at
** your option) any later version.
**
********************************************************/
#ifndef QyurSqlTreeView_H
#define QyurSqlTreeView_H
#include <QAbstractProxyModel>
#include <QSqlRecord>
#include <QTreeView>

class QSqlTableModel;
class QyurSqlTreeModelPrivate;
class QyurSqlTreeModel: public QAbstractProxyModel {
	Q_OBJECT
	Q_DECLARE_PRIVATE(QyurSqlTreeModel)
	Q_DISABLE_COPY(QyurSqlTreeModel)
	Q_CLASSINFO("Author", "Yuri Yurachkovsky")
	Q_CLASSINFO("e-mail", "yura_79@mail.ru")
	Q_CLASSINFO("Version", "0.93")
	Q_CLASSINFO("Qt version", "4.3.3")
	QyurSqlTreeModelPrivate* d_ptr;
public:
	/*
	tableName-		table name itself,
	captions-		friendly captions, must match to fieldNames,
	fieldNames-		original field names in table. all fields in tables must be specified,
	idField-		original id field name,
	parentIdField	original parentId field name,
	rootParentId	parentId value for top level nodes
	*/
	QyurSqlTreeModel(const QString& tableName, const QStringList& captions, const QStringList& fieldNames, const QString& idField, const QString& parentIdField, int rootParentId= 0, QObject* parent=0);
	~QyurSqlTreeModel();
	//void setSourceModel(QAbstractItemModel* sourceModel);
	QModelIndex mapFromSource(const QModelIndex& sourceIndex) const;
	QModelIndex mapToSource(const QModelIndex& proxyIndex) const;
	bool hasChildren(const QModelIndex& parent=QModelIndex()) const;
	int columnCount(const QModelIndex& parent=QModelIndex()) const;
	QModelIndex index(int row, int column, const QModelIndex& parent=QModelIndex()) const;
	QModelIndex parent(const QModelIndex& index) const;
	int rowCount(const QModelIndex& parent=QModelIndex()) const;
	bool setData(const QModelIndex& index, const QVariant& value, int role =Qt::EditRole);
	void sort(int column, Qt::SortOrder order=Qt::AscendingOrder);
	QVariant headerData(int section, Qt::Orientation orientation, int role=Qt::DisplayRole) const;
	const QSqlTableModel* getSourceModel() const;
	int getIdByIndex(const QModelIndex&) const;
	QModelIndex getIndexById(int id) const;
	int getParidByIndex(const QModelIndex&) const;
	int getSourceIdColumn() const;
	int getSourceParentIdColumn() const;
	bool removeRecords(const QModelIndexList&);
	int getRootParentId() const;
	int originalColumnByProxy(int proxyColumn) const;
	int proxyColumnByOriginal(int originalColumn) const;
	int proxyColumnByOriginal(const QString& field) const;
	QModelIndex addChild(const QVariantList& values, const QModelIndex& parent, QAbstractItemView*);
	QModelIndexList expandedList(QObject*) const;
	QModelIndexList selectionList(QObject*) const;
	QModelIndex current(QObject*) const;
signals:
	void beforeRefresh();
	void afterRefresh();
public slots:
	void refresh();
	// This slots can be connected to expanded() and collapsed() signals from TreeView.
	// This use to restore TreeView expanded nodes after refreshing of underlying SqlTableModel.
	void addExpanded(const QModelIndex&);
	void addCollapsed(const QModelIndex&);
	//This slots can be connected to selectionChanged() and currentChanged() of QItemSelectionModel.
	//This use to restore TreeView selected nodes after refreshing of underlying SqlTableModel.
	void onSelectionChanged(const QItemSelection&, const QItemSelection&);
	void onCurrentChanged(const QModelIndex&);
};

class QyurSqlTreeViewPrivate;
class QyurSqlTreeView: public QTreeView {
	Q_OBJECT
	Q_DECLARE_PRIVATE(QyurSqlTreeView)
	Q_DISABLE_COPY(QyurSqlTreeView)
	Q_CLASSINFO("Author", "Yuri Yurachkovsky")
	Q_CLASSINFO("e-mail", "yura_79@mail.ru")
	Q_CLASSINFO("Version", "0.93")
	Q_CLASSINFO("Qt version", "4.3.3")
	QyurSqlTreeViewPrivate* d_ptr;
public:
	QyurSqlTreeView();
	~QyurSqlTreeView();
	void setModel(QAbstractItemModel* model);
	void setColumnHidden(const QString& column, bool hide); 
	int columnIndex(const QString& fieldName) const;
	//values matches source table columns order
	QModelIndex addChild(const QVariantList& values, const QModelIndex& parent= QModelIndex());
	void remove(const QModelIndexList& list);
protected slots:
	//Use this handler after table repopulated
	virtual void onAfterRefresh();
};

#endif
